@@ -1,6 +1,6 @@
 package Smil;
 
-$VERSION = "0.72";
+$VERSION = "0.85";
 
 use Carp;
 use SMIL::XMLBase;
@@ -8,6 +8,7 @@ use SMIL::XMLContainer;
 use SMIL::XMLTag;
 use SMIL::Head;
 use SMIL::Body;
+
 my $TRUE = 'true';
 
 @ISA = qw( SMIL::XMLContainer );
@@ -29,6 +30,13 @@ my $QT_SHORT = 'qt';
 my $QT_NS_URL = "http://www.apple.com/quicktime/resources/smilextensions";
 my $QT_NS = "xmlns:qt";
 
+my $SMIL_2_NS = 'xmlns';
+my $SMIL_2_NS_URL = 'http://www.w3.org/2001/SMIL20/Language';
+
+my $RN_SMIL_2_NAMESPACE = 'rnSmil2';
+my $RN_SMIL_2_NS_URL = "http://features.real.com/2001/SMIL20/Extensions";
+my $RN_SMIL_2_NS = "xmlns:$RN_SMIL_2_NAMESPACE";
+
 my $VERSION_COMPATIBLE = 'version';
 my $PLAYER_COMPATIBLE = 'player';
 my $RP_COMPATIBLE = 'rp';
@@ -54,10 +62,10 @@ my @RP_VERSION_MAPPING = ( "",  		# 0
 																													"", 		# 6, no switch allowed
 																													
 																													# 7, Gold Player 7
-																													"http://features.real.com/?feature;player=6.0.7.380", 		
+																													"http://features.real.com/?component;player=6.0.7.380", 		
 																													
 																													# 8, Gold Player 8
-																													"http://features.real.com/?feature;player=6.0.9.450",
+																													"http://features.real.com/?component;player=6.0.9.450",
 																													);
 
 my %VERSION_MAPPING = ( $RP_COMPATIBLE => \@RP_VERSION_MAPPING );
@@ -73,24 +81,29 @@ my @smilAttributes = ( $RN_NS,
 																							$QT_NEXT,
 																							$QT_TIME_SLIDER );
 
+my @smil2Extensions = ( 'syncBehavior' );
+my @rnSmil2Extensions = ( 'backgroundOpacity', 'bgcolor', 'chromaKey', 
+																										'chromaKeyOpacity', 'chromaKeyTolerance', 'mediaOpacity' );
+
+
 sub init {
     my $self = shift;
     $self->SUPER::init( "smil" );
-
+				
     my %hash = @_;
     my %smilAttrs = $self->createValidAttributes( { %hash },
-						  [ @smilAttributes ] );
-
+																																																		[ @smilAttributes ] );
+				
     if( $hash{ $QT_SHORT } or 
-	$hash{ $QT_EXTENSIONS } ) {
-	$self->useQtExtensions;
+								$hash{ $QT_EXTENSIONS } ) {
+								$self->useQtExtensions;
     }
-
+				
     if( $hash{ $RN_EXTENSIONS } or 
-	$hash{ $RN_SHORT } ) {
-	$self->useRnExtensions;	
+								$hash{ $RN_SHORT } ) {
+								$self->useRnExtensions;	
     }
-
+				
     $self->setAttributes( %smilAttrs );
     
     $self->setFavorite( $favorite ) if $favorite;
@@ -100,49 +113,49 @@ sub init {
 }
 
 sub setQtAutoplay {
-	my $self = shift;
-	$self->setAttribute( $QT_AUTOPLAY => $TRUE );
-	$self->useQtExtensions;
+				my $self = shift;
+				$self->setAttribute( $QT_AUTOPLAY => $TRUE );
+				$self->useQtExtensions;
 }
 
 sub setQtChapterMode {
-	my $self = shift;
-	my $dur = shift;
-	$self->setAttribute( $QT_CHAPTER_MODE => $dur );
-	$self->useQtExtensions;
+				my $self = shift;
+				my $dur = shift;
+				$self->setAttribute( $QT_CHAPTER_MODE => $dur );
+				$self->useQtExtensions;
 }
 
 sub useQtImmediateInstantiation {
-	my $self = shift;
-	$self->setAttribute( $QT_IMMEDIATE_INSTANTIATION => $TRUE );
-	$self->useQtExtensions;
+				my $self = shift;
+				$self->setAttribute( $QT_IMMEDIATE_INSTANTIATION => $TRUE );
+				$self->useQtExtensions;
 }
 
 sub setQtNextPresentation {
-	$self = shift;
-	$next = shift;
-	$self->setAttribute( $QT_NEXT => $next );
-	$self->useQtExtensions;
+				$self = shift;
+				$next = shift;
+				$self->setAttribute( $QT_NEXT => $next );
+				$self->useQtExtensions;
 }
 
 sub useQtTimeSlider {
-	$self = shift;
-	$self->setAttribute( $QT_TIME_SLIDER => $TRUE );
-	$self->useQtExtensions;
+				$self = shift;
+				$self->setAttribute( $QT_TIME_SLIDER => $TRUE );
+				$self->useQtExtensions;
 }
 
 sub useRnExtensions {
-	my $self = shift;
-	my $version = shift;
-
-	$self->setAttribute( $RN_NS => $RN_NS_URL );
-	$self->setFavorite( $RN_NS );
+				my $self = shift;
+				my $version = shift;
+				
+				$self->setAttribute( $RN_NS => $RN_NS_URL );
+				$self->setFavorite( $RN_NS );
 }
 
 sub useQtExtensions {
-	my $self = shift;
-	$self->setAttribute( $QT_NS => $QT_NS_URL );
-	$self->setFavorite( $QT_NS );
+				my $self = shift;
+				$self->setAttribute( $QT_NS => $QT_NS_URL );
+				$self->setFavorite( $QT_NS );
 }
 
 sub getRootHeight {
@@ -170,7 +183,7 @@ sub initFile {
     my $self = shift;
     my %hash = @_;
     if( $hash{ $file } ) {
-	$self->{$file} = $hash{ $file };
+								$self->{$file} = $hash{ $file };
     }
 }
 
@@ -178,8 +191,8 @@ sub initHead {
     my $self = shift;
     my %hash = @_;
     $self->setTagContents( $head => new SMIL::Head( @_ ) )
-	if( ( $hash{ 'height' } && $hash{ 'width' } ) ||
-	    $hash{ 'meta' } );
+								if( ( $hash{ 'height' } && $hash{ 'width' } ) ||
+												$hash{ 'meta' } );
 }
 
 sub initBody {
@@ -208,26 +221,92 @@ sub endSequence {
 }
 
 sub hasQtExtensions {
-
-   my $self = shift;
-   my %hash = @_;
-   my $returnValue = 0;
-
-   foreach my $item ( keys %hash ) {
-	$returnValue = 1 if $item =~ /^qt:/;
+				
+				my $self = shift;
+				my %hash = @_;
+				my $returnValue = 0;
+				
+				foreach my $item ( keys %hash ) {
+								$returnValue = 1 if $item =~ /^qt:/;
    }
-
-   return $returnValue;
+				
+				return $returnValue;
 }
 
-sub addMedia {
-    my $self = shift;
+sub hasRnSmil2Extensions { 
+				my $self = shift;
+				
+				my $returnValue = 0;
+				
+				my %the_hash = @_;
+				
+				foreach my $item ( keys %the_hash ) {
+								foreach my $rnAttribute ( @rnSmil2Extensions ) {
+												$returnValue = 1 if $item eq $rnAttribute;
+								}
+				}
+				
+				return $returnValue;
+}
+
+sub useRnSmil2Extensions { 
+				my $self = shift;
+				$self->setAttribute( $RN_SMIL_2_NS => $RN_SMIL_2_NS_URL );
+}
+
+sub hasSmil2Extensions { 
+				my $self = shift;
+
+				my $returnValue = 0;
+
+				my %the_hash = @_;
+				
+				foreach my $item ( keys %the_hash ) {
+								foreach my $rnAttribute ( @smil2Extensions ) {
+												$returnValue = 1 if $item eq $rnAttribute;
+								}
+				}
+
+				return $returnValue;
+}
+
+sub useSmil2Extensions { 
+				my $self = shift;
+				$self->setAttribute( $SMIL_2_NS => $SMIL_2_NS_URL );
+}
+
+sub checkForExtensions {
+				
+				my $self = shift;
 				
     # Check for QT extensions, and add if necessary
     if( $self->hasQtExtensions( @_ ) ) {
 								$self->useQtExtensions;
     }
 				
+				# Check for SMIL 2.0 attributes
+				if( $self->hasSmil2Extensions( @_ ) ) {
+								$self->useSmil2Extensions;
+				}
+
+				# Check for RN Smil 2.0 extensions
+				if( $self->hasRnSmil2Extensions( @_ ) ) {
+								$self->useRnSmil2Extensions;
+				}
+
+}
+
+sub addInlinedMedia {
+				my $self = shift;
+
+				$self->addMedia( @_, inline => 1 );
+}
+
+sub addMedia {
+    my $self = shift;
+
+				$self->checkForExtensions( @_ );
+				
     # Make sure that if we are adding inline that we
     # add the RP version checking code in case
     # we need to add a switch because we are authoring
@@ -279,7 +358,7 @@ sub addBackwardsCompatibleSwitch {
 				my $feature = shift;
     my @medias = ();
 				
-    my $earliestVersion = &getEarliestVersionForAttribute( $attribute ); 
+    my $earliestVersion = &getEarliestVersionForAttribute( $feature ); 
 				# $self->getPrivate( $VERSION_COMPATIBLE ); # Don't really need this right now..
     my $playerType = $self->getPrivate( $PLAYER_COMPATIBLE );
 				
@@ -289,13 +368,13 @@ sub addBackwardsCompatibleSwitch {
 				
 				# Remove the attribute
 				my %withoutSwitchingAttribute = @_;
-				undef( $withoutSwitchingAttribute{ $attribute } );
+				undef( $withoutSwitchingAttribute{ $feature } );
 				
     # Create a different media object for each of the versions
     # to support.  Make sure to go backwards since switch uses first
     # match
 				my $supportedIndex = &getEarliestSupportedVersion( $playerType, $feature );
-
+				
 				my @attributes = ( %withSwitchingAttribute, 
 																							( @RP_SYSTEM_REQUIRED_ATTRIBUTE, 
 																									&getSupportedVersionAttribute( $supportedIndex, 	
@@ -342,6 +421,27 @@ sub isInlined {
 				return defined( $args{ $INLINE } );
 }
 
+sub addTransition {
+				my $self = shift;
+				# Need to add the transition to the head.
+				$self->useSmil2Extensions;
+				my $head_ref;
+				if( !( $head_ref = $self->getContentObjectByName( $head ) ) ) {
+								$head_ref = new SMIL::Head();
+								# Head must go at the top of the items in the <smil>
+								$self->unshiftTagContents( $head => $head_ref );
+				}
+				$head_ref->addTransition( @_ );
+}
+
+sub addAnimation {
+				my $self = shift;
+
+				$self->useSmil2Extensions;
+
+				die "addAnimation NYI";
+}
+
 sub addCode {
     my $self = shift;
     $self->getContentObjectByName( $body )->addCode( @_ );
@@ -362,13 +462,17 @@ sub addSwitchedMedia {
 sub setSwitchedLayout {
     my $self = shift;
     if( $self->{$head} ) {
-	$self->setTagContents( $head => new SMIL::Head( @_ ) );
+								$self->setTagContents( $head => new SMIL::Head( @_ ) );
     }
     $self->getContentObjectByName( $head )->setSwitchedLayout( @_ );
 }
 
 sub header {
-    return "Content-type: application/smil\n\n";
+    return "Content-type: " . &getMimeType() . "\n\n";
+}
+
+sub getMimeType {
+				return "application/smil";
 }
 
 sub setMeta {
@@ -396,8 +500,9 @@ Smil.pm - Perl extension for dynamic generation of SMIL files.
 
 =head1 SYNOPSIS
 
-  use Smil;
-  (read on, good knight...)
+ use Smil;
+$s = new Smil;
+$s->getAsString;
 
 =head1 DESCRIPTION
 
@@ -405,7 +510,7 @@ This module provides an object oriented interface to generation of
 SMIL (Synchronized Multimedia Integration Language) files.  Creating
 a simple SMIL file is as simple as this script:
 
-    use Smil;
+ use Smil;
 $s = new Smil;
 print $s->header; # Remember MIME header if running as CGI
 $s->addMedia( "src" => "rtsp://videoserver.com/video.rm" );
@@ -649,6 +754,149 @@ inline.pl file inside the installation directory, and also the slurp.pl
 script which will slurp in simple SMIL files and inline all code if you give
 it the proper parameter.
 
+=head1 METHODS 
+
+This is a comprehensive listing of all the methods exposed by Smil.pm.
+
+=head2 Basic Smil functionality
+
+addMedia(): this method adds a media reference or other to a SMIL 
+document.  It is best to read the above documentation on addMedia since there
+are many parameters available.  You should always have a "src" key/value
+pair at the very least.  "src" can be a reference to a local file, 
+a reference to a remote file (http:// or rtsp://, for example), or 
+it can be a object reference, for any object which supports the 
+getAsString() and getMimeType() method interface.  RN::RealText is one 
+example, but any one can write their own class and use it here.  Peruse 
+the sample scripts in the installation directory for examples.
+
+addRegion(): this method adds a region for media playback.  The only
+required attribute is "name" (or "id" which is synonymous) which uniquely
+identifies the region, and should be a name starting with a letter, 
+composed of alphanumeric characters.  For SMIL files which will be played
+back in a SMIL 1.0 compliant player you should also use the attributes
+"height" and "width".  "z-index" specifies the stacking order of the media
+regions and is optional.  Refer to the documentation above for more 
+information on addRegion, and peruse the sample scripts in the installation
+directory for other examples.
+
+getAsString(): this method returns the SMIL file as a string.  It takes no
+parameters.  Once you have finished composing your SMIL file, you 
+retrieve the code with this method.
+
+startSequence(): this method starts a sequence of media references, or 
+grouping of media references, where each piece of media will be played 
+in succession, one after the other.  Compare to startParallel.  It takes
+no parameters.
+
+startParallel(): this method starts a grouping of media references where
+each piece of media will be played with the same beginning time.  Compare
+to startSequence.  It takes no parameters.
+
+endParallel(): end a media group.  Every call to startParallel() should
+be matched by a call to endParallel, although if all media have been
+added this call can be safely omitted and Smil.pm will add the proper
+number of closing <par> tags.  It takes no parameters.
+
+endSequence(): end a media sequence.  Every call to startSequence() should
+be matched by a call to endSequence, although if all media have been
+added this call can be safely omitted and Smil.pm will add the proper
+number of closing <seq> tags.  It takes no parameters.
+
+addCode(): this method allows arbitrary code snippets in SMIL.  If a 
+feature of SMIL is not supported by the exposed methods of the Smil.pm
+module, code for that feature can be added using this method.  It takes
+a string as its only parameter.
+
+addComment(): this method allows commenting in SMIL files.  It takes a 
+string as its only parameter.
+
+getRootHeight(): convenience method to retrieve the height of the SMIL
+file.
+
+getRootWidth(): convenience method to retrieve the width of the SMIL
+file.
+
+header(): convenience method which returns the MIME type for SMIL;
+useful when the perl script is running as a CGI program.
+
+=head2 Advanced Smil functionality
+
+addInlinedMedia(): this method has the same parameter options as addMedia()
+but it automatically generates the extra parameters to inline the media 
+directly within the SMIL file.
+
+addSwitchedMedia(): this method allows a user to add several pieces of 
+media to SMIL such that one piece will be displayed in one configuration
+or version of a media player, and another piece of media will be displayed
+in another.
+
+setSwitchedLayout(): this method adds a switched layout to SMIL, so 
+that one player version or player configuration can display a different layout
+than another player version or player configuration.
+
+setBackwardsCompatible(): call this method if you want the Smil.pm module
+to automatically generate SMIL code which is backwards compatible.  In 
+general this method is useful for small SMIL files, or prototyping, 
+because the module is not especially smart about optimizing switch 
+statements.  You must call this with two parameters, "player" => <name> and
+"version" => <number>.  Right now only "player" => "rp" RealPlayer) is 
+supported for backwards compatibility, since it is the only player I know of
+which has players with different compatibility levels for SMIL.
+
+=head2 RealNetworks extensions
+
+useRnExtensions(): call this method to guarantee that the RealNetworks 
+namespace will be added to the <smil> tag.  Normally this method
+will be called automatically for you when you use SMIL features
+that require it, but if you add your own code and need to use this
+namespace it is available.
+
+=head2 Quicktime extensions
+
+setQtAutoplay(): this method tells the Quicktime player to automatically
+start playing the presentation once the SMIL has been loaded.  Refer
+to the Quicktime documetation for more details on this feature.
+
+setQtChapterMode(): Refer to the Quicktime documentation for more details
+on this feature.
+
+useQtImmediateInstantiation(): Refer to the Quicktime documentation for 
+more details on this feature.
+
+setQtNextPresentation():  this method tells the Quicktime player to load
+another presentation once this SMIL presentation is finished.  Refer 
+to the Quicktime documentation for more details on this feature.
+
+useQtTimeSlider(): this method tells the Quicktime player to enable the
+time slider.  Refer to the Quicktime documentation for more details 
+on this feature.
+
+hasQtExtensions(): this method returns true or false depending on whether
+the SMIL document is using Quicktime extensions.
+
+useQtExtensions(): this method will normally be called when one of the
+above methods is used.  It adds the quicktime namespace to the SMIL
+document so that Quicktime extensions can be used.
+
+=head2 SMIL 2.0 extensions
+
+addTransition(): this method adds a transition to the SMIL file.  
+
+addAnimation(): this method is currently under development for SMIL 2.0
+extensions.
+
+useRnSmil2Extensions(): this method adds a RN specific namespace to the 
+SMIL file which can be used to add RealNetworks SMIL 2.0 extensions. 
+It is most often called internally by the Smil.pm module when it encounters
+media which attempt to use RealNetworks SMIL 2.0 extensions.
+
+useSmil2Extensions():  this method adds the SMIL 2.0 namespace to the SMIL
+file which allows a SMIL renderer/player to verify that this code has SMIL 
+2.0 features.  It is most often called internally by the Smil.pm module
+when it encounters media references which attempt to use generic SMIL 2.0 
+extensions.
+
 =head1 AUTHOR
 
 Chris Dawson (cdawson@webiphany.com)
@@ -656,6 +904,6 @@ http://www.webiphany.com/perlysmil/
 
 =head1 SEE ALSO
 
-perl(1). perldoc CGI
+perl(1). perldoc RN::RealText, perldoc CGI 
 
 =cut
